// -*- c++ -*-
//------------------------------------------------------------------------------
// $Id: CardBox.cpp,v 1.35 2004/07/21 03:57:46 vlg Exp $
//------------------------------------------------------------------------------
//                            CardBox.cpp
//------------------------------------------------------------------------------
//  Copyright (c) 2004 by Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//------------------------------------------------------------------------------
//
// Date   : Sat Apr 17 23:37:11 EDT 2004
//
//------------------------------------------------------------------------------

#include <limits.h>				// PATH_MAX
#include <libgen.h>				// basename(3), dirname(3)
#include <string.h>				// strcpy(3)

#include <gtkmm/table.h>
#include <gtkmm/eventbox.h>

#include "Granule.h"
#include "CardBox.h"
#include "Config.h"
#include "MainWindow.h"
#include "CardRef.h"
#include "Deck.h"

CardBox::
CardBox ()
{
	trace_with_mask("CardBox::CardBox",GUITRACE);
	Gtk::EventBox* evtbox;
	int i;
	
	m_drawer_empty = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_empty.xpm");
	m_drawer_one   = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_one.xpm");
	m_drawer_many  = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/drawer_many.xpm");
	m_sched_green  = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_green.xpm");
	m_sched_grey   =
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_grey.xpm");
	m_sched_red    = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_red.xpm");
	m_sched_yellow = 
		Gdk::Pixbuf::create_from_file(GRAPPDATADIR "/pixmaps/sched_yellow.xpm");

	for (i = 0; i < CARD_BOX_SIZE; i++) {
		m_cbs.push_back (CardBoxIF ());
		m_cbs [i].m_sched_light = manage (new Gtk::Image);
		m_cbs [i].m_sched_count = manage (new Gtk::Label);
		m_cbs [i].m_sched_vbox  = manage (new Gtk::VBox (false, 0));
		m_cbs [i].m_box_pic  = manage (new Gtk::Image);
		m_cbs [i].m_total_count = manage (new Gtk::Label);

		m_cbs [i].m_sched_light->set (m_sched_grey);
		m_cbs [i].m_sched_light->set_alignment (0.5,0.5);
		m_cbs [i].m_sched_light->set_padding (0,2);

		m_cbs [i].m_sched_count->set_alignment(0.5,0.5);
		m_cbs [i].m_sched_count->set_padding(0,0);
		m_cbs [i].m_sched_count->set_justify(Gtk::JUSTIFY_CENTER);
		m_cbs [i].m_sched_count->set_line_wrap(false);
		m_cbs [i].m_sched_count->set_text ("0");

		m_cbs [i].m_sched_vbox->pack_start (
			*m_cbs [i].m_sched_light, Gtk::PACK_SHRINK, 2);

		m_cbs [i].m_sched_vbox->pack_start (
			*m_cbs [i].m_sched_count, Gtk::PACK_SHRINK, 0);

		m_cbs [i].m_box_pic->set (m_drawer_empty);
		m_cbs [i].m_box_pic->set_alignment (0.5,0.5);
		m_cbs [i].m_box_pic->set_padding (2,2);
		
		m_cbs [i].m_total_count->set_alignment (0,0.5);
		m_cbs [i].m_total_count->set_padding (8,0);
		m_cbs [i].m_total_count->set_justify (Gtk::JUSTIFY_RIGHT);
		m_cbs [i].m_total_count->set_line_wrap (false);
		m_cbs [i].m_total_count->set_text ("0 cards");
	}

	Gtk::Table* cbt = manage (new Gtk::Table (5, 3, false));
	
	cbt->set_row_spacings(1);
	cbt->set_col_spacings(1);

	/** Also enable card box icon as DND destination.
	 */
	std::list<Gtk::TargetEntry> list_targets;
	list_targets.push_back (Gtk::TargetEntry ("STRING"));
	list_targets.push_back (Gtk::TargetEntry ("text/plain"));

	for (i = 0; i < 5; i++) {
		evtbox = manage (new Gtk::EventBox);
		evtbox->add (*(m_cbs [i].m_box_pic));
		evtbox->set_events (Gdk::BUTTON_PRESS_MASK);

		evtbox->signal_button_press_event ().connect (
			sigc::bind<int>(sigc::mem_fun(*this, &CardBox::play_in_box_cb), i));

		evtbox->drag_dest_set (list_targets);
		evtbox->signal_drag_data_received ().connect (
			sigc::bind(sigc::mem_fun (*this, &CardBox::on_deck_dropped_cb), i));

		cbt->attach (*evtbox, 1,2,i,i+1, Gtk::FILL, Gtk::FILL, 0,0);
	}

	for (i = 0; i < 5; i++) {
		cbt->attach (*m_cbs[i].m_total_count, 
					 2,3,i,i+1, 
					 Gtk::FILL,
					 Gtk::AttachOptions(), 
					 0,0);
	}

	for (i = 0; i < 5; i++) {
		cbt->attach (*m_cbs [i].m_sched_vbox, 0,1,i,i+1, 
					 Gtk::FILL, Gtk::FILL,4,0);
	}
  	pack_start (*cbt, Gtk::PACK_SHRINK, 4);
	show_all ();
}

CardBox::
~CardBox ()
{
	trace_with_mask("CardBox::~CardBox",GUITRACE);
}

void
CardBox::
update_count (int idx_)
{
	std::ostringstream os;
	
	os << m_card_deck [idx_].size () << " cards";
	m_cbs [idx_].m_total_count->set_text (os.str ());

	m_cbs [idx_].m_box_pic->clear ();	
	if (m_card_deck [idx_].size () == 0) {
		m_cbs [idx_].m_box_pic->set (m_drawer_empty);
	}
	else if (m_card_deck [idx_].size () == 1) {
		m_cbs [idx_].m_box_pic->set (m_drawer_one);
	}
	else {
		m_cbs [idx_].m_box_pic->set (m_drawer_many);
	}
}

void
CardBox::
new_file ()
{
	trace_with_mask("CardBox::new_file",GUITRACE);

	m_fname = UNKNOWN_FNAME;
	CONFIG->set_proj_name (m_fname);
	for (int i = 0; i < CARD_BOX_SIZE; i++) {
		m_card_deck [i].set_name (i);
		m_cdplayers [i] = new DeckPlayer (m_card_deck [i]);
	}
	MAINWIN->set_mw_name ();
}

int
CardBox::
load (const string& fname_)
{
	trace_with_mask("CardBox::load",GUITRACE);

	/** Test to see if file exists
	 */
    struct stat file_stat;
    if (stat (fname_.c_str (), &file_stat) < 0 ||
		S_ISREG (file_stat.st_mode) == false) // invalid project file
	{
		string msg ("Invalid CardBox file:\n");
		msg += fname_;
		Gtk::MessageDialog md_error (msg, false, Gtk::MESSAGE_ERROR);
		return -1;
    }

    CONFIG->set_proj_name (fname_);

	try {
		xmlpp::DomParser parser;
		parser.set_validate ();
		parser.parse_file (fname_);
		if (!parser) {
			DL((GRAPP,"Failed to parse \"%s\"\n", fname_.c_str ()));
			return -1;
		}
		DL((GRAPP,"Parsing the card file ...\n"));
		const xmlpp::Node* root_node = parser.get_document ()->get_root_node ();
		Assert_exit (root_node);

		if (parse_xml_tree (root_node) == 0) {
			m_fname = fname_;
			MAINWIN->set_mw_name ();
			CONFIG->set_filesel_path (fname_);
			return 0;
		}
	}
	catch (const std::exception& ex_) {
		string msg ("Failed to parse file\n   ");
		msg += fname_ + "\nReason: invalid file syntax/DTD";
		Gtk::MessageDialog e (msg, false, MESSAGE_ERROR);
		e.run ();
	}
	return -1;
}

int
CardBox::
parse_xml_tree (const xmlpp::Node* node_)
{
	trace_with_mask("CardBox::parse_xml_tree",GUITRACE);
	char xpath [64];
	char total_count [32];
	const xmlpp::TextNode* text_node;
	const xmlpp::Attribute* attr_node;
	xmlpp::NodeSet nset;		// set of cards
	xmlpp::NodeSet::iterator nset_iter;
	long card_id;
	string deck_path;
	int ccount;					// card counter
	int cloaded;				// count of cards loaded
	Card* cardptr;
	CardRef* card_ref;

	for (int i = 0; i < CARD_BOX_SIZE; i++) {
		DL((GRAPP,"Filling Card Deck # %d ...\n", i+1));
		m_card_deck [i].set_name (i);
		cloaded = 0;
		for (ccount = 1; ccount < MAX_CARDS; ccount++) {
			DL((GRAPP,"Parsing Card # %d/%d ...\n", i+1, ccount));
			sprintf(xpath, "/cardfile/carddeck[%d]/card[%d]", i+1, ccount);
			nset = node_->find (xpath);
			if ((nset_iter = nset.begin ()) == nset.end ()) {
				break;
			}
			/** Parse card's ID
			 */
			sprintf (xpath, "/cardfile/carddeck[%d]/card[%d]/@id", i+1, ccount);
			nset = node_->find (xpath);
			if ((nset_iter = nset.begin ()) == nset.end ()) return -1;
			attr_node = dynamic_cast<const xmlpp::Attribute*> (*nset_iter);
			if (attr_node == NULL) {
				return -1;
			}
			card_id = ::atol ((attr_node->get_value ().substr (1)).c_str ());

			/** Parse the expiration time of the card (expdate tag). If
				tag is not found, then we are dealing with an older file.
			 */
			ASSA::TimeVal exp_time;
			sprintf (xpath, "/cardfile/carddeck[%d]/card[%d]/expdate/text()", 
					 i+1, ccount);
			nset = node_->find (xpath);
			if ((nset_iter = nset.begin ()) == nset.end ()) {
				exp_time = CONFIG->get_load_time (); // card expires today
			}
			else {
				text_node = dynamic_cast<const xmlpp::TextNode*> (*nset_iter);
				if (text_node == NULL) {
					return -1;
				}
				exp_time.sec (::atol (text_node->get_content ().c_str ()));
			}

			/** Parse the source Deck path of the card
			 */
			sprintf (xpath, "/cardfile/carddeck[%d]/card[%d]/fromdeck/text()", 
					 i+1, ccount);
			nset = node_->find (xpath);
			if ((nset_iter = nset.begin ()) == nset.end ()) {
				return -1;
			}
			text_node = dynamic_cast<const xmlpp::TextNode*> (*nset_iter);
			if (text_node == NULL) {
				return -1;
			}
			deck_path = Glib::locale_from_utf8 (text_node->get_content ());

			DL((GRAPP,"Card parsed ID=%ld,\n path = \"%s\"\n", 
				card_id, deck_path.c_str ()));

			/** Look up the card in the list of decks by (ID, PATH) pair.
				If deck is found, it is loaded (if it is not already in the
				memory) and a newly created CardRef object is returned.
			 */
			card_ref = DECKMGR->lookup_card (card_id, deck_path);
			if (card_ref != NULL) {
				DL((GRAPP,"Card (ID=%ld) found\n", card_id));
				card_ref->set_expiration (exp_time);
				if (card_ref->is_expired ()) {
					m_cbs [i].add_one ();
				}
				m_card_deck [i].push_back (card_ref);
				cloaded++;
			}
		} // for (each card) 

		m_card_deck [i].sort_by_expiration_time ();
		sprintf (total_count, "%d cards", cloaded);
		update_count (i);
		set_expiration_light (i);
		m_cdplayers [i] = new DeckPlayer (m_card_deck [i]);

	} // for (each card box)

	return 0;
}
	
int
CardBox::
save ()
{
	trace_with_mask("CardBox::save",GUITRACE);

	xmlpp::Document deck_xml;
	xmlpp::Element* deck_node;
	xmlpp::Element* carddeck;
	xmlpp::Element* card;
	xmlpp::Element* path;
	xmlpp::TextNode* txt_node;

	VDeck::cardlist_iterator citer;
	CardRef* card_ref;

	DL((GRAPP,"Saving to \"%s\"\n", m_fname.c_str ()));
	::unlink (m_fname.c_str ());
	deck_xml.set_internal_subset ("cardfile", "", GRAPPXMLDIR "/cardfile.dtd");
	deck_node = deck_xml.create_root_node ("cardfile");

	for (int i = 0; i < CARD_BOX_SIZE; i++) {
		carddeck = deck_node->add_child ("carddeck");
		citer = m_card_deck [i].begin ();
		while (citer != m_card_deck [i].end ()) {
			if ((card_ref = dynamic_cast<CardRef*> (*citer)) != NULL) {
				card = carddeck->add_child ("card");
				card->set_attribute ("id", (*citer)->get_id_str ());

				path = card->add_child ("fromdeck");
				txt_node = path->add_content ();
				txt_node->set_content (
					Glib::locale_to_utf8 (card_ref->get_deck ().get_name ()));

				path = card->add_child ("expdate");
				txt_node = path->add_content ();
				txt_node->set_content (card_ref->get_expiration_str ());
			}
			citer++;
		}
	}
	deck_xml.write_to_file_formatted (m_fname, "UTF-8");
}

int
CardBox::
save_as (const string& fname_)
{
	trace_with_mask("CardBox::save_as",GUITRACE);

	/** If extension is not given, add it to the filepath.
	 */
	m_fname = fname_;
    string ext (CARD_FILE_EXT);
    if (m_fname.find (ext) == string::npos) {
		m_fname += ext;
    }

	if (Glib::file_test (fname_, Glib::FILE_TEST_IS_REGULAR)) {
		string msg = "File \n" + fname_ 
			+ "\nalready exists.\nWould you like to delete it first?";
		Gtk::MessageDialog qm (*MAINWIN, 
							   msg, 
							   false,
							   Gtk::MESSAGE_QUESTION, 
							   Gtk::BUTTONS_NONE,
							   true);
		qm.add_button ("Yes", Gtk::RESPONSE_YES);
		qm.add_button ("No",  Gtk::RESPONSE_NO );
		if (qm.run () == Gtk::RESPONSE_NO) {
			return -1;
		}
		::unlink (fname_.c_str ());
	}
	save ();
	CONFIG->set_proj_name (fname_);
}

void
CardBox::
close ()
{
	trace_with_mask("CardBox::close",GUITRACE);

	for (int i = 0; i < CARD_BOX_SIZE; i++) {
		delete m_cdplayers [i];
		m_cdplayers [i] = NULL;
		m_card_deck [i].clear ();
		m_cbs [i].reset ();
		m_cbs [i].m_box_pic->clear ();
		m_cbs [i].m_box_pic->set (m_drawer_empty);
		m_cbs [i].m_sched_light->set (m_sched_grey);
	}
	m_fname = "";				// No cardbox is loaded any more
}

void
CardBox::
play_deck (int idx_)
{
	trace_with_mask("CardBox::play_deck",GUITRACE);
	
	if (idx_ < 0 || idx_ > CARD_BOX_SIZE-1) {
		Gtk::MessageDialog w ("Trying to access index out of range!", false,
							  MESSAGE_ERROR);
		w.run ();
	}
	if (m_card_deck [idx_].size ()) 
	{
		m_card_deck [idx_].reset_progress ();
		m_cdplayers [idx_]->run ();
		update_count (idx_);	// User might have deleted some cards.
	}
	else {
		Gtk::MessageDialog w ("Card Deck you've selected is empty", false,
							  MESSAGE_WARNING);
		w.run ();
	}
}

/** 
	Add all cards from_deck_ to the CardDeck number idx_.
	This method is called from DeckMgr::add_deck_to_box (id) callback.
 */
void
CardBox::
add_deck_to_box (int idx_, Deck* from_deck_)
{
	trace_with_mask("CardBox::add_deck_to_box",GUITRACE);
	Assert_exit (idx_ < CARD_BOX_SIZE);
	Assert_exit (from_deck_);

	if (m_fname.empty ()) {
		Gtk::MessageDialog em ("First, open or create\na Card Box file.", false,
							   Gtk::MESSAGE_ERROR);
		em.run ();
		return;
	}
	
	CardRef* card_ref;
	VDeck::cardlist_const_iterator iter = from_deck_->begin ();

	while (iter != from_deck_->end ()) {
		card_ref = new CardRef (**iter, *from_deck_);
		card_ref->set_expiration (CONFIG->get_expiration (idx_));
		m_card_deck [idx_].push_back (card_ref);
		iter++;
	}
	if (CONFIG->get_remove_duplicates ()) {
		m_card_deck [idx_].remove_duplicates ();
	}
	m_card_deck [idx_].reset_progress ();
	update_count (idx_);
	m_cdplayers [idx_]->reset ();
	MAINWIN->go_state (Dirty);
}

/**
   Add card from DeckView to the CardBox. This function can be called 
   both from DeckView to add selected card to CardBox #1 or when 
   a new card is created and the configuration option 
   { new_card_to_cardbox } is set to true.

   @param card_ref_ New card reference to add. Assume ownership.
*/
void 
CardBox::
add_card_to_box (CardRef* card_ref_)
{
	trace_with_mask("CardBox::add_card_to_box",GUITRACE);

	if (m_fname.empty ()) {
		Gtk::MessageDialog em ("First, open or create a Card File.", false,
							   Gtk::MESSAGE_ERROR);
		em.run ();
		return;
	}
	card_ref_->set_expiration (CONFIG->get_expiration (0));
	m_card_deck [0].push_back (card_ref_);
	m_card_deck [0].reset_progress ();
	update_count (0);
	m_cdplayers [0]->reset ();
	MAINWIN->go_state (Dirty);
}

/** Move a card from one Card Box to another.
	This method is invoked by the DeckPlayer (know_answer_cb(), no_clue_cb())
	when the user is either guesses the card right or not.
*/
void
CardBox::
move_card_to_box (VDeck::cardlist_iterator card_ref_, 
				  int from_deck_, 
				  int to_deck_,
				  bool is_expired_)
{
	trace_with_mask("CardBox::move_card_to_box",GUITRACE);

	if (to_deck_ >= CARD_BOX_SIZE) {
		to_deck_ = CARD_BOX_SIZE - 1;
	}

	/** Get hold of CardRef itself
	 */
	VCard* vcard = *card_ref_;

	/** First, remove the card from the old CardDeck.
	 */ 
	DL((GRAPP,"Removing card from the old CardDeck ...\n"));
	m_card_deck [from_deck_].erase (card_ref_);
	update_count (from_deck_);

	if (is_expired_) {
		m_cbs [from_deck_].m_expired_count--;
		set_expiration_light (from_deck_);
		if (m_cbs [from_deck_].m_expired_count == 0) {
			Gtk::MessageDialog msg ("You have learnt all\n"
									"expired cards in this deck.", false,
									MESSAGE_INFO);
			msg.run ();
		}
	}

	if (to_deck_ != from_deck_) {
		m_cdplayers [from_deck_]->reset ();
	}

	/** Insert the CardRef in the back of the next CardDeck.
		In case of the same CardDeck (#1 if 'don't know' and #5 if 'know it'),
		the card goes back to the end of the CardDeck.
	*/
	DL((GRAPP,"Placing card in next/first CardDeck ...\n"));
	vcard->set_expiration (CONFIG->get_expiration (to_deck_));
	m_card_deck [to_deck_].push_back (vcard);
	if (to_deck_ != from_deck_) {
		m_card_deck [to_deck_].reset_progress ();
	}
	m_cdplayers [to_deck_]->reset ();
	update_count (to_deck_);
	set_expiration_light (to_deck_);

	MAINWIN->go_state (Dirty);
}

bool 
CardBox::
play_in_box_cb (GdkEventButton* event, int idx_) 
{
	trace_with_mask("CardBox::play_in_box_cb",GUITRACE);

	if (m_fname.empty ()) {
		Gtk::MessageDialog emsg ("First Open or create\na New Card File!", 
								 false, MESSAGE_ERROR);
		emsg.run ();
	}
	else {
		MAINWIN->play_in_box (idx_);
		set_expiration_light (idx_);
	}
	return true;
}

/** Set the expiration light according to the ratio of expired
	cards to the total count:

        [0%;   25%[ - grey; 
        [25%;  50%[ - green; 
        [50%;  75%[ - yellow;
        [75%; 100%] - red.

	@param idx_ Index of the CardDeck
*/
void 
CardBox::
set_expiration_light (int idx_) 
{
	trace_with_mask("CardBox::set_expiration_light",GUITRACE);

	std::ostringstream os;
	os << m_cbs [idx_].m_expired_count;
	m_cbs [idx_].m_sched_count->set_text (os.str ());
	
	if (m_card_deck [idx_].size () == 0) {
		m_cbs [idx_].m_sched_light->set (m_sched_grey);
		return;
	}

	float x = (m_cbs [idx_].m_expired_count * 100.0) /
		m_card_deck [idx_].size();

	DL((GRAPP,"m_expired_count/deck size : %d/%d (x = %f)\n",
		m_cbs [idx_].m_expired_count, m_card_deck[idx_].size(), x)); 

	if (x >= 0 && x < 25.0) {                                  // grey
		m_cbs [idx_].m_sched_light->set (m_sched_grey);
	}
	else if (x >= 25.0 && x < 50.0) {                          // green
		m_cbs [idx_].m_sched_light->set (m_sched_green);
	}
	else if (x >= 50.0 && x < 75.0) {                          // yellow
		m_cbs [idx_].m_sched_light->set (m_sched_yellow);
	}
	else {				                                       // red
		m_cbs [idx_].m_sched_light->set (m_sched_red);
	}
}

void 
CardBox::
on_deck_dropped_cb (const Glib::RefPtr<Gdk::DragContext>& context_,
					int drop_x_coordinate_,
					int drop_y_coordinate_,
					const Gtk::SelectionData& selection_data_,
					guint type_id_,
					guint drop_timestamp_,
					int card_deck_index_)
{
	std::ostringstream os;
	Deck* src_deck;

	trace_with_mask("CardBox::on_deck_dropped_cd",GUITRACE);
	
    /** Doesn't work - I get bogus pointer back! (investigate)
	    Deck* src_deck = (Deck*)(selection_data_.get_data ());
	*/

	src_deck = MAINWIN->get_dnd_source ();

	os << "Proceed with adding Deck \n\"" << src_deck->get_name () << "\"\n"
	   << " to Card Box # " << card_deck_index_ << " ?";

	Gtk::MessageDialog qm (*MAINWIN,
						   os.str (),
						   false,
						   Gtk::MESSAGE_QUESTION, 
						   Gtk::BUTTONS_NONE,
						   true);
	qm.add_button ("Yes", Gtk::RESPONSE_YES);
	qm.add_button ("No",  Gtk::RESPONSE_NO );
	if (qm.run () == Gtk::RESPONSE_NO) {
		return;
	}

	DL((GRAPP,"Adding Deck \"%s\" to CardBox # %d\n", 
		src_deck->get_name ().c_str (), card_deck_index_));
		
	add_deck_to_box (card_deck_index_, src_deck);
	context_->drag_finish (false, false, drop_timestamp_);
}
