// -*- c++ -*-
//------------------------------------------------------------------------------
//                              CardView.cpp
//------------------------------------------------------------------------------
// $Id: CardView.cpp,v 1.9 2004/07/06 04:34:23 vlg Exp $
//
//  Copyright (c) 2004 by Vladislav Grinchenko
//
//  This program is free software; you can redistribute it and/or 
//  modify it under the terms of the GNU General Public License   
//  as published by the Free Software Foundation; either version  
//  2 of the License, or (at your option) any later version.      
//
// Date: Feb 15, 2004
//------------------------------------------------------------------------------

#include <gtkmm/image.h>
#include <gtkmm/label.h>
#include <gtkmm/box.h>
#include <gtkmm/alignment.h>
#include <gtkmm/button.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/table.h>

#include "Config.h"
#include "CardView.h"
#include "Card.h"
#include "CardRef.h"

CardView::
CardView (VCard& card_) : 
	m_card (card_),
	m_cancel_button (NULL),
	m_ok_button (NULL),
	m_back_text (NULL),
	m_example_text (NULL),
	m_front_text (NULL)
{
	trace_with_mask("CardView::CardView",GUITRACE);

	Gtk::Image *image1;
	Gtk::Label *label7;
	Gtk::HBox *hbox2;
	Gtk::Alignment *alignment1;
	Gtk::Image *image2;
	Gtk::Label *label8;
	Gtk::HBox *hbox3;
	Gtk::Alignment *alignment2;
	Gtk::Image *image3;
	Gtk::Label *label9;
	Gtk::HBox *hbox4;
	Gtk::Alignment *alignment3;
	Gtk::ScrolledWindow *backtext_scrollwin;
	Gtk::Label *back_label;
	Gtk::Alignment *alignment5;
	Gtk::VBox *vbox_backlabel;
	Gtk::Label *example_label;
	Gtk::VBox *vbox_examplelabel;
	Gtk::ScrolledWindow *example_scrollwin;
	Gtk::Label *front_label;
	Gtk::Alignment *alignment6;
	Gtk::VBox *vbox_frontlabel;
	Gtk::Alignment *alignment4;
	Gtk::Table *elems_table;
	Gtk::VBox *vbox_all;

	set_title ("Card View");
	set_default_size (643, 386); // 3x5 ratio
	set_modal (true);
	property_window_position ().set_value (Gtk::WIN_POS_NONE);
	set_resizable (true);
	property_destroy_with_parent ().set_value (false);
	set_has_separator (true);

	image1 = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-clear"), 
										Gtk::IconSize (4)));
	label7 = Gtk::manage (new Gtk::Label ("Clear"));
	hbox2 = Gtk::manage (new Gtk::HBox (false, 2));
	alignment1 = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
	image2 = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-cancel"), 
										Gtk::IconSize (4)));
	label8 = Gtk::manage (new Gtk::Label ("Cancel"));
	hbox3 = Gtk::manage (new Gtk::HBox (false, 2));
	alignment2 = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
	m_cancel_button = Gtk::manage (new Gtk::Button ());
	image3 = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-apply"), 
										Gtk::IconSize (4)));
	label9 = Gtk::manage (new Gtk::Label ("Ok"));
	hbox4 = Gtk::manage (new Gtk::HBox (false, 2));
	alignment3 = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
	m_ok_button = Gtk::manage (new Gtk::Button ());
	m_back_text = Gtk::manage (new Gtk::TextView ());
	backtext_scrollwin = Gtk::manage (new Gtk::ScrolledWindow ());
	back_label = Gtk::manage (new Gtk::Label ("Back :"));
	alignment5 = Gtk::manage (new Gtk::Alignment (0.76, 0.5, 0, 0));
	vbox_backlabel = Gtk::manage (new Gtk::VBox (false, 0));
	example_label = Gtk::manage (new Gtk::Label ("Example : "));
	vbox_examplelabel = Gtk::manage (new Gtk::VBox (false, 0));
	m_example_text = Gtk::manage (new Gtk::TextView ());
	example_scrollwin = Gtk::manage (new Gtk::ScrolledWindow ());
	m_front_text = Gtk::manage (new Gtk::Entry ());
	front_label = Gtk::manage (new Gtk::Label ("Front : "));
	alignment6 = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
	vbox_frontlabel = Gtk::manage (new Gtk::VBox (false, 0));
	alignment4 = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
	elems_table = Gtk::manage (new Gtk::Table (2, 2, false));
	vbox_all = Gtk::manage (new Gtk::VBox (false, 0));

	m_front_text->modify_font (CONFIG->get_font ());
	m_back_text->modify_font (CONFIG->get_font ());
	m_example_text->modify_font (CONFIG->get_font ());

	image1->set_alignment (0.5,0.5);
	image1->set_padding (0,0);
	label7->set_alignment (0.5,0.5);
	label7->set_padding (0,0);
	label7->set_justify (Gtk::JUSTIFY_LEFT);
	label7->set_line_wrap (false);
	label7->set_use_markup (false);
	label7->set_selectable (false);

	hbox2->pack_start (*image1, Gtk::PACK_SHRINK, 0);
	hbox2->pack_start (*label7, Gtk::PACK_SHRINK, 0);
	alignment1->add (*hbox2);

	image2->set_alignment (0.5,0.5);
	image2->set_padding (0,0);
	label8->set_alignment (0.5,0.5);
	label8->set_padding (0,0);
	label8->set_justify (Gtk::JUSTIFY_LEFT);
	label8->set_line_wrap (false);
	label8->set_use_markup (false);
	label8->set_selectable (false);
	hbox3->pack_start (*image2, Gtk::PACK_SHRINK, 0);
	hbox3->pack_start (*label8, Gtk::PACK_SHRINK, 0);
	alignment2->add (*hbox3);

	m_cancel_button->set_flags (Gtk::CAN_FOCUS);
	m_cancel_button->set_relief (Gtk::RELIEF_NORMAL);
	m_cancel_button->add (*alignment2);

	image3->set_alignment (0.5,0.5);
	image3->set_padding (0,0);
	label9->set_alignment (0.5,0.5);
	label9->set_padding (0,0);
	label9->set_justify (Gtk::JUSTIFY_LEFT);
	label9->set_line_wrap (false);
	label9->set_use_markup (false);
	label9->set_selectable (false);
	hbox4->pack_start (*image3, Gtk::PACK_SHRINK, 0);
	hbox4->pack_start (*label9, Gtk::PACK_SHRINK, 0);
	alignment3->add (*hbox4);

	m_ok_button->set_flags (Gtk::CAN_FOCUS);
	m_ok_button->set_relief (Gtk::RELIEF_NORMAL);
	m_ok_button->add (*alignment3);

	get_action_area ()->property_layout_style ().set_value (Gtk::BUTTONBOX_END);
	m_back_text->set_flags (Gtk::CAN_FOCUS);
	m_back_text->set_border_width (3);
	m_back_text->set_editable (true);
	m_back_text->set_cursor_visible (true);
	m_back_text->set_pixels_above_lines (3);
	m_back_text->set_pixels_below_lines (0);
	m_back_text->set_pixels_inside_wrap (0);
	m_back_text->set_left_margin (0);
	m_back_text->set_right_margin (0);
	m_back_text->set_indent (0);
	m_back_text->set_wrap_mode (Gtk::WRAP_WORD);
	m_back_text->set_justification (Gtk::JUSTIFY_LEFT);
	m_back_text->get_buffer ()->set_text ("Fill up the back of the card");

	backtext_scrollwin->set_flags (Gtk::CAN_FOCUS);
	backtext_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
	backtext_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
	backtext_scrollwin->
		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
	backtext_scrollwin->add (*m_back_text);

	back_label->set_alignment (0.5,0.5);
	back_label->set_padding (0,0);
	back_label->set_justify (Gtk::JUSTIFY_LEFT);
	back_label->set_line_wrap (false);
	back_label->set_use_markup (false);
	back_label->set_selectable (false);
	alignment5->add (*back_label);
	vbox_backlabel->pack_start (*alignment5, Gtk::PACK_SHRINK, 0);

	example_label->set_alignment (0.5,0.5);
	example_label->set_padding (0,0);
	example_label->set_justify (Gtk::JUSTIFY_RIGHT);
	example_label->set_line_wrap (false);
	example_label->set_use_markup (false);
	example_label->set_selectable (false);
	vbox_examplelabel->pack_start (*example_label, Gtk::PACK_SHRINK, 0);

	m_example_text->set_flags (Gtk::CAN_FOCUS);
	m_example_text->set_border_width (3);
	m_example_text->set_editable (true);
	m_example_text->set_cursor_visible (true);
	m_example_text->set_pixels_above_lines (3);
	m_example_text->set_pixels_below_lines (0);
	m_example_text->set_pixels_inside_wrap (0);
	m_example_text->set_left_margin (0);
	m_example_text->set_right_margin (0);
	m_example_text->set_indent (0);
	m_example_text->set_wrap_mode (Gtk::WRAP_WORD);
	m_example_text->set_justification (Gtk::JUSTIFY_LEFT);
	m_example_text->get_buffer ()->set_text ("Fill up an example");

	example_scrollwin->set_flags (Gtk::CAN_FOCUS);
	example_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
	example_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
	example_scrollwin->
		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
	example_scrollwin->add (*m_example_text);

	m_front_text->set_flags (Gtk::CAN_FOCUS);
	m_front_text->set_visibility (true);
	m_front_text->set_editable (true);
	m_front_text->set_max_length (0);
	m_front_text->set_text ("Fill up the front of the card");
	m_front_text->set_has_frame (true);
	m_front_text->set_activates_default (false);

	front_label->set_alignment (0.5,0.5);
	front_label->set_padding (0,0);
	front_label->set_justify (Gtk::JUSTIFY_LEFT);
	front_label->set_line_wrap (false);
	front_label->set_use_markup (false);
	front_label->set_selectable (false);
	alignment6->add (*front_label);
	vbox_frontlabel->pack_start (*alignment6, Gtk::PACK_SHRINK, 0);

	elems_table->set_border_width (4);
	elems_table->set_row_spacings (8);
	elems_table->set_col_spacings (2);

	elems_table->attach (*backtext_scrollwin, 1, 2, 1, 2, 
						Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
	elems_table->attach (*vbox_backlabel, 0, 1, 1, 2, 
						Gtk::FILL, Gtk::FILL, 0, 0);
	elems_table->attach (*vbox_examplelabel, 0, 1, 2, 3, 
						Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
	elems_table->attach (*example_scrollwin, 1, 2, 2, 3, 
						Gtk::FILL, Gtk::FILL, 0, 0);
	elems_table->attach (*m_front_text, 1, 2, 0, 1, 
						Gtk::EXPAND|Gtk::FILL, Gtk::AttachOptions (), 0, 1);
	elems_table->attach (*vbox_frontlabel, 0, 1, 0, 1, 
						Gtk::FILL, Gtk::FILL, 0, 0);
	elems_table->attach (*alignment4, 0, 1, 3, 4, Gtk::FILL, Gtk::FILL, 0, 0);
	vbox_all->pack_start (*elems_table);

	get_vbox ()->set_homogeneous (false);
	get_vbox ()->set_spacing (0);
	get_vbox ()->pack_start (*vbox_all);

	add_action_widget (*m_cancel_button, Gtk::RESPONSE_CANCEL);
	add_action_widget (*m_ok_button,     Gtk::RESPONSE_OK);

	m_ok_button->signal_clicked ().connect (
		sigc::mem_fun (*this, &CardView::on_ok_clicked));

	/** Fill up the form
	 */
	m_front_text->set_text (m_card.get_question ());
	m_back_text->get_buffer ()->set_text  (m_card.get_answer ());
	m_example_text->get_buffer ()->set_text (m_card.get_example ());

	show_all();
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
//                              Callbacks
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

/** Fend off clicking Esc key on the keyboard.
	This often happens when I want to enter ~ (tilda) key, and instead
	press Esc. The effect is to close the editing window which leads to
	the loss of all data already entered. The only way to close dialog
	is to click on Cancel button.

	@param event_ Event data structure.
*/
bool
CardView::
on_key_press_event (GdkEventKey* event_)
{
	trace_with_mask("CardView::on_key_press_event",GUITRACE);
	DL((GRAPP,"key_pressed = %d\n", event_->keyval));

	/** Trap and disable ESC key event.
		The values for all keysyms are found 
		in $prefix/gtk-2.0/gdk/gdkkeysyms.h
	*/
	if (event_->keyval == GDK_Escape) { 
		return false;
	}
	return Dialog::on_key_press_event (event_);	// keep processing keystroke
}
	   

void
CardView::
on_ok_clicked  ()
{
	trace_with_mask("CardView::on_ok_clicked",GUITRACE);

	m_card.set_question (m_front_text->get_text ());
	m_card.set_answer (m_back_text->get_buffer ()->get_text ());
	m_card.set_example (m_example_text->get_buffer ()->get_text ());
}

